/*******************************************************************************
 *   $FILE:  main.c
 *   Atmel Corporation:  http://www.atmel.com \n
 *   Support email:  touch@atmel.com
 ******************************************************************************/

/*  License
 *   Copyright (c) 2010, Atmel Corporation All rights reserved.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   3. The name of ATMEL may not be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 *   WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
 *   SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
 *   INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *   THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*----------------------------------------------------------------------------
  compiler information
  ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  include files
  ----------------------------------------------------------------------------*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#define __delay_cycles(n)     __builtin_avr_delay_cycles(n)
#define __enable_interrupt()  sei()

#include "touch_api.h"
#include "touch.h"
#include "lcd.h"
#include "util/delay.h"
/*----------------------------------------------------------------------------
  manifest constants
  ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  macros
  ----------------------------------------------------------------------------*/

#define GET_SENSOR_STATE(SENSOR_NUMBER) qt_measure_data.qt_touch_status.sensor_states[(SENSOR_NUMBER/8)] & (1 << (SENSOR_NUMBER % 8))
#define GET_ROTOR_SLIDER_POSITION(ROTOR_SLIDER_NUMBER) qt_measure_data.qt_touch_status.rotor_slider_values[ROTOR_SLIDER_NUMBER]

/*----------------------------------------------------------------------------
  type definitions
  ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  prototypes
  ----------------------------------------------------------------------------*/
extern void touch_measure();
extern void touch_init( void );
extern void init_system( void );
extern void init_timer_isr(void);
extern void set_timer_period(uint16_t);
/*----------------------------------------------------------------------------
  Structure Declarations
  ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  macros
  ----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
  global variables
  ----------------------------------------------------------------------------*/
/* Timer period in msec. */
uint16_t qt_measurement_period_msec = QT_MEASUREMENT_PERIOD_MS;
uint16_t time_ms_inc=0;
/*----------------------------------------------------------------------------
  extern variables
  ----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  static variables
  ----------------------------------------------------------------------------*/

/* flag set by timer ISR when it's time to measure touch */
volatile uint8_t time_to_measure_touch = 0u;

/* current time, set by timer ISR */
volatile uint16_t current_time_ms_touch = 0u;

/*============================================================================
Name    :   main
------------------------------------------------------------------------------
Purpose :   main code entry point
Input   :   n/a
Output  :   n/a
Notes   :
============================================================================*/

#define temp_max 33
#define temp_min 21

void IO_init()
{
    DDRD |= (1 << PD7); /* Led User */
	
	DDRA |= (1 << PA4); /* Backlight LCD */
	PORTA |= (1 << PA4);
	
    DDRD |= (1 << PD2); /* Led User*/
    /* butoanele sunt puse ca intrare, cu rezistenta de pull-up */
    DDRD &= ~(1 << PD0); /* buton */
    PORTD |= (1 << PD0);

    DDRD &= ~(1 << PD6); /* buton */
    PORTD |= (1 << PD6);

    DDRD &= ~(1 << PD1); /*buton */
    PORTD |= (1 << PD1);

}
void adc_init()
{
    // AREF = AVcc
    ADMUX = (1<<REFS0);

    // ADC Enable and prescaler of 128
    
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
}

// read adc value
uint16_t adc_read(uint8_t ch)
{
    // select the corresponding channel 0~7
   
    ch &= 0b00000111;  // AND operation with 7
    ADMUX = (ADMUX & 0xF8)|ch;     

    // start single conversion
    // write '1' to ADSC
    ADCSRA |= (1<<ADSC);

    // wait for conversion to complete
    // ADSC becomes '0' again
    
    while(ADCSRA & (1<<ADSC));

    return (ADC);
}


void timer1_init()
{
    /* timer1 este configurat cu modul FastPWM, top ICR1 si fara prescaler */
    TCCR1A = (1 << COM1B1) | (0 << COM1B0) | (1 << WGM11) | (0 << WGM10);
    TCCR1B = (1 << ICES1) | (1 << WGM13) | (1 << WGM12) | (1 << CS10);
    
    TCNT1 = 0;
    ICR1 = 639;
    OCR1B = ICR1 /4 ;
    DDRD |= (1 << PD4);
}


void automatic_control(int temp)
{
    if (temp <= temp_min )
        OCR1B = 0;
    else
    if (temp >= temp_max)
        OCR1B = ICR1;
    else
    OCR1B = (float) ICR1 * ( temp  - temp_min ) /(temp_max - temp_min);
}
void touch_control(uint8_t value)
{

    if (value == 0)
        OCR1B = 0;
    else
       OCR1B = (float)(value / 30.f) * ICR1;
}
int main( void )
{
    uint16_t adc_result0;
    float temp;
    int t1,t2;
    char buffer1[16];
    char buffer2[16];
    char mode = 'T'; /*A automtic, M manual */
    uint16_t lastvalue = 0;
    uint8_t value = 0;

    /* initialise host app, pins, watchdog, etc */
    init_system();

    /* configure timer ISR to fire regularly */
    init_timer_isr();

    /* Initialize Touch sensors */
    touch_init();

    /* Input-Output initialization */
    IO_init();

    /* LCD initialization */
    LCD_init();

    /* Timer 1 initialization */
    timer1_init();

    /* Analog to digital convertor intialization */
    adc_init();
	
    /* loop forever */
    for( ; ; )
    {   
        adc_result0 = adc_read(3);      // read adc value at PA3                       
        temp=adc_result0/2.0;
        /************************************ LCD part **************************************/
        LCD_writeInstr(LCD_INSTR_clearDisplay);
        t1 = (int) temp;
        t2 = (int) ((temp - t1)*10);
        /* line 1 */
        sprintf(buffer1,"Temp:%d.%dC Mod:%c",t1,t2,mode);
        LCD_printAt(0x00,buffer1);
        /* line 2 */
        int a = (OCR1B*100)/ICR1;
        if ( a < 0)
            a = 100 - (-1) * a;
        sprintf(buffer2,"Duty: %d",(int)a);
        LCD_printAt(0x40,buffer2);

        if ((PIND & (1 << PD6)) == 0) {
            if (mode == 'A')
                mode = 'B';
            else
            if (mode == 'B')
                mode = 'T';
            else
            if (mode == 'T')
                mode = 'A';
            PORTD |= (1 << PD7);
        }
        /********************************* Automatic Control **********************************/
        if (mode == 'A')
            automatic_control(t1);
        else
         /********************************* QTouch Control ***********************************/
        if(mode =='T'){
            MCUCR |= (1u << PUD); //MCUCR_PUD = 1u;  
            touch_measure();
            MCUCR &= ~(1u << PUD); //MCUCR_PUD = 0u;

            if (qt_measure_data.qt_touch_status.rotor_slider_values[0] != 0)
            {
                uint16_t newvalue = qt_measure_data.qt_touch_status.rotor_slider_values[0];
                if (newvalue > lastvalue && newvalue < (lastvalue+ 10))
                {
                    if (value >0) value = (value - 1) ;
                }
                if (newvalue < lastvalue && newvalue > (lastvalue - 10))
                {
                    if ( value < 30) value = (value + 1) ;
                    }
                    
                touch_control(value);
                lastvalue = newvalue;
            }
        }
         /********************************* Buttons Control **********************************/
        else{           
            if ((PIND & (1 << PD1)) == 0) {
                if((int) OCR1B - (int)ICR1/32 <=0)
                    OCR1B = 0;
                else
                    OCR1B-=ICR1 / 32;
                PORTD |= (1 << PD2);
            }
            if ((PIND & (1 << PD0)) == 0) {
                if ( OCR1B + ICR1/32 >= ICR1)
                    OCR1B = ICR1;
                else 
                    OCR1B+=ICR1 / 32;
                PORTD |= (1 << PD2);
            }
        }

        _delay_ms(10);
        PORTD &= ~(1 << PD2);
        PORTD &= ~(1 << PD7);
        

        _delay_ms(100);
    }
}

